# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

constrained_stress <- function(x, W, D) {
    .Call(`_graphlayouts_constrained_stress`, x, W, D)
}

constrained_stress_major <- function(y, dim, W, D, iter, tol) {
    .Call(`_graphlayouts_constrained_stress_major`, y, dim, W, D, iter, tol)
}

constrained_stress3D <- function(x, W, D) {
    .Call(`_graphlayouts_constrained_stress3D`, x, W, D)
}

constrained_stress_major3D <- function(y, dim, W, D, iter, tol) {
    .Call(`_graphlayouts_constrained_stress_major3D`, y, dim, W, D, iter, tol)
}

reweighting <- function(el, N_ranks) {
    .Call(`_graphlayouts_reweighting`, el, N_ranks)
}

sparseStress <- function(y, D, Rp, pivots, A, maxIter) {
    .Call(`_graphlayouts_sparseStress`, y, D, Rp, pivots, A, maxIter)
}

stress <- function(x, W, D) {
    .Call(`_graphlayouts_stress`, x, W, D)
}

stress_major <- function(y, W, D, iter, tol) {
    .Call(`_graphlayouts_stress_major`, y, W, D, iter, tol)
}

stress_radii <- function(y, W, D, r, tseq) {
    .Call(`_graphlayouts_stress_radii`, y, W, D, r, tseq)
}

stress_focus <- function(y, W, D, Z, tseq, iter, tol) {
    .Call(`_graphlayouts_stress_focus`, y, W, D, Z, tseq, iter, tol)
}

stress3D <- function(x, W, D) {
    .Call(`_graphlayouts_stress3D`, x, W, D)
}

stress_major3D <- function(y, W, D, iter, tol) {
    .Call(`_graphlayouts_stress_major3D`, y, W, D, iter, tol)
}

